var Editor = function (editorElement, htmlEditor, textEditor) {
	this.init(editorElement, htmlEditor, textEditor);
}

Editor.prototype = {

  normalMode  : 1,
  sourceMode  : 3,

  init : function(editorElement, htmlEditor, textEditor) // constructor
  {
	this.editorElement = document.getElementById(editorElement);
	this.richEditor = document.getElementById(htmlEditor);
	this.textEditor = document.getElementById(textEditor);
	this._loadRichEditor();
	this._loadTextEditor();
  }, // init()

  viewHtml : function()
  {
    globalState.title = document.getElementById("doctitle").value;
    globalState.description = document.getElementById("docDescription").value;
    if (this.editorShell.IsHTMLSourceMode() == true) {
	  var textContent = this.content;
	  this.editorElement.setAttribute('selectedIndex', '0');
	  this.editorShell.SetDisplayMode(this.normalMode);
	  this.content = textContent;
	} else {
	  this.editorElement.setAttribute('selectedIndex', '0');
	}
	this.focus();
  },

  viewSource : function()
  {
    globalState.title = document.getElementById("doctitle").value;
    globalState.description = document.getElementById("docDescription").value;
	if (this.editorShell.IsHTMLSourceMode() == false) { // show source
	  var textContent = this.content;
	  this.editorElement.setAttribute('selectedIndex','1');
	  this.editorShell.SetDisplayMode(this.sourceMode);
	  this.content = textContent;
	} else {
	  this.editorElement.setAttribute('selectedIndex','1');
	}
	this.focus();
  },

  viewMetadata : function()
  {
    var currView = document.getElementById('discussEditorType');
    if (currView.getAttribute('selectedIndex') != 3) {
      globalState.content = this.content;
      //document.getElementById("docCategory");
      currView.setAttribute('selectedIndex', 3);
    }
  },

  updateMetadata : function()
  {
    var docTitle = document.getElementById("doctitle");
    if (!docTitle.value || docTitle.value == "")
      docTitle.value = this.generateSummary(this.content, 100);
    var docDesc = document.getElementById("docDescription")
    if (!docDesc.value || docDesc.value == "")
      docDesc.value = this.generateSummary(this.content, 500);
  },

  dumpState : function()
  {
    globalState.content = this.content;
    globalState.title = document.getElementById("doctitle").value;
    globalState.description = document.getElementById("docDescription").value;
  },

  clearMeta : function()
  {
    document.getElementById("doctitle").value = "";
    document.getElementById("docDescription").value = "";
  },

  generateSummary : function(tmpContent, length)
  {
    var whitespace = /\s+/gi;
    var htmlTag = /<[^<>]*>/gi;
    tmpContent = tmpContent.replace(whitespace, " ");
    tmpContent = tmpContent.replace(htmlTag, "");
    tmpContent = tmpContent.slice(0, length);
    var lastFullStopIndex = tmpContent.lastIndexOf(".");
    if (lastFullStopIndex > 0)
      tmpContent = tmpContent.slice(0,lastFullStopIndex+1);
    return tmpContent;
  },

  get isNormalMode() // check if its in html mode
  {
	return !this.editorShell.IsHTMLSourceMode();
  },

  get content()  // get the content of the editor
  {
	var content = this.editorShell.IsHTMLSourceMode() ?
	  this.textEditor.value :
	  this.editorShell.GetContentsAs('text/html', 4);

	if (this.editorShell.GetEditMode() == this.normalMode) {
	  content = content.replace(/^<.+<body>/gi, "");
	  content = content.replace(/<\/body><\/html>/gi, "");
	}

	dump('mode: ' + this.editorShell.GetEditMode() + ', HTMLMode: ' +
	  this.editorShell.IsHTMLSourceMode() + "\n");
	dump('content: ' + content + "\n");

	return content;
  },

  dumpText : function()
  {
	alert(this.editorShell.GetContentsAs('text/plain', 2));
  },

  set content(contentText)	// set the content for the editor
  {
	if (this.editorShell.IsHTMLSourceMode() == false) { // it's graphical editor
		var editorContent = '<html><head></head><body>' + contentText + "</body></html>";
		// bug <br> inserted if it's the last focused element, use \n hack to get around it
		this.editorShell.RebuildDocumentFromSource(editorContent);
	} else {
		this.textEditor.value = contentText;
		this.textEditor.setAttribute('hidden','true');
		this.textEditor.setAttribute('hidden','false');
	}
    this.updateMetadata();
  },

  focus : function()
  {
    if(this.isNormalMode)
	  this.richEditor.focus();
    else
	  this.textEditor.focus();
  },

  ///////////////////////////////////////////////////
  // private functions
  ///////////////////////////////////////////////////
  _loadRichEditor : function ()
  {
	// setup the rich editor
	this.editorShell = this.richEditor.editorShell;
	window.editorShell = this.editorShell; // for external dependancies like image upload
	this.editorShell.editorType = 'html';
    //SetupHTMLEditorCommands();

	for (var wCount = 0; wCount < window.length; wCount++) {   // find the window instance bottombar is in
	  if (window.frames[wCount].name == this.richEditor.getAttribute('name')) {
		this.editorShell.webShellWindow = window.frames[wCount];
		this.editorShell.contentWindow = window.frames[wCount];
	  }
	}
	this.editorShell.RegisterDocumentStateListener(DocumentStateListener);
	this.editorShell.contentsMIMEType = 'text/html';
	this.editorShell.LoadUrl('about:blank');
  }, // _loadRichEditor

  _loadTextEditor : function ()
  {
	var sourceWordwrap = null;
	try {	 // setup the editor is it will wrap if defined in preference
	  sourceWordwrap = discussPref.getBoolPref('discuss.sourceWordwrap');
	} catch (ex) {
	  sourceWordwrap = false;
	}

	if (sourceWordwrap) {
	  this.textEditor.setAttribute('cols','1');
      document.getElementById('docDescription').setAttribute('cols','1');
	} else {
	  this.textEditor.removeAttribute('cols');
      document.getElementById('docDescription').removeAttribute('cols');
	}
  } // _loadTextEditor
};

var DocumentStateListener =
{
  NotifyDocumentCreated: function()
  {
    if (tmpStore.exists())
      tmpStore.load();
    else
      tmpStore.save();

    window.updateCommands("create");
/* Potential future code for selection handling
	addEditorListener();
	editorHandler = editorHandle.editorShell.editor.QueryInterface(Components.interfaces.nsIEditor);
	selectionHandle = editorHandler.selection.QueryInterface(Components.interfaces.nsISelection);
	selectionHd2 = selectionHandle.QueryInterface(Components.interfaces.nsISelectionPrivate);
	selectionHd2.addSelectionListener(selectListener);
	*/
  },
  NotifyDocumentWillBeDestroyed: function() {},
  NotifyDocumentStateChanged:function( isNowDirty )
  {
    window.updateCommands("create");
    window.updateCommands("save");
  }
};

/* Potential future code for selection handling

function addEditorListener()
{
	var editor = editorHandle.editorShell.editor.QueryInterface(Components.interfaces.nsIEditor);
	editor.addEditActionListener(DocumentActionListener);
}

var selectListener = {
	notifySelectionChanged: function(doc, selection, reason) {}
};

var DocumentActionListener = {
	WillCreateNode: function(tag, parent, position) {},
	DidCreateNode: function(tag, node, parent, position, result) {},
	WillInsertNode: function(node, parent, position) {},
	DidInsertNode: function(node, parent, position, result) {},
	WillDeleteNode: function(child) {},
	DidDeleteNode: function(child, result) {},
	WillSplitNode: function(existingRightNode, offset) {},
	DidSplitNode: function(existingRightNode, offset, newLeftNode, result) {},
	WillJoinNodes: function(leftNode, rightNode, parent) {},
	DidJoinNodes: function(leftNode, rightNode, parent, result) {},
	WillInsertText: function(textNode, offset, string) {},
	DidInsertText: function(textNode, offset, string, result) {},
	WillDeleteText: function(textNode, offset, length) {},
	DidDeleteText: function(textNode, offset, length, result) {},
	WillDeleteSelection: function(selection) {},
	DidDeleteSelection: function(selection) {}
};
*/